<?php

/**
 * This source file is subject to the GNU GENERAL PUBLIC LICENSE (GPL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.gnu.org/licenses/gpl-3.0.txt
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
} // Exit if accessed directly

if( !class_exists( 'venice_lite_admin_notice' ) ) {

	class venice_lite_admin_notice {
	
		/**
		 * Constructor
		 */
		 
		public function __construct( $fields = array() ) {

			if ( 
				!get_option( 'venice-lite-hidden-notice') &&
				version_compare( PHP_VERSION, VENICE_LITE_MIN_PHP_VERSION, '>=' )
			) {

				add_action( 'admin_notices', array(&$this, 'admin_notice') );
				add_action( 'admin_head', array( $this, 'dismiss' ) );
			
			}

		}

		/**
		 * Dismiss notice.
		 */
		
		public function dismiss() {

			if ( isset( $_GET['venice-lite-dismiss'] ) && check_admin_referer( 'venice-lite-dismiss-action' ) ) {
		
				update_option( 'venice-lite-hidden-notice', intval($_GET['venice-lite-dismiss']) );
				remove_action( 'admin_notices', array(&$this, 'admin_notice') );
				
			} 
		
		}

		/**
		 * Admin notice.
		 */
		 
		public function admin_notice() {
			
		?>
			
            <div class="notice notice-warning is-dismissible">
            
            	<p>
            
            		<strong>

						<?php
                        
							esc_html_e( 'Pay what you want for purchasing the Venice theme and receive an additional bundle of 13 WordPress products ', 'venice-lite' ); 
                        
							printf( 
								' <em>%s</em><br/>', 
								esc_html__( '(Minimum price allowed €1)', 'venice-lite' )
							);

							printf( 
								'<a href="%1$s" class="dismiss-notice">' . esc_html__( 'Dismiss this notice', 'venice-lite' ) . '</a>', 
								esc_url( wp_nonce_url( add_query_arg( 'venice-lite-dismiss', '1' ), 'venice-lite-dismiss-action'))
							);

                        ?>
                    
                    </strong>
                    
            	</p>
                    
            	<p>
            		
                    <a target="_blank" href="<?php echo esc_url( 'https://www.themeinprogress.com/venice-free-woocommerce-wordpress-blog-theme/?ref=2&campaign=venice-notice' ); ?>" class="button button-primary"><?php esc_html_e( 'Name your price', 'venice-lite' ); ?></a>
                
            	</p>

            </div>
		
		<?php
		
		}

	}

}

new venice_lite_admin_notice();

?>